<?php
namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Teknisa\Entities;
use OrganizationsManagement\Util\Utilities;
use Util\Messages;
use OrganizationsManagement\Util;
use OrganizationsManagement\Util\InstanceProvider;
use Teknisa\Model\Geral\Repositories as GeralRepositories;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;
use Teknisa\Model\Geral\Entities\Organizacao as EntidadeOrganizacao;
use Teknisa\Model\Teknisa\Entities\OmParamOrg as EntidadeOmParamOrg;

class ScoreReport
{

    const TYPE_ISSUE        = Entities\CpCall::TYPE_ISSUE;
    const TYPE_CASO         = Entities\CpCall::TYPE_CASO;
    const TYPE_RICH_CLIENT  = Entities\CpCall::TYPE_RICH_CLIENT;
    const TYPE_TICKET       = Entities\CpCall::TYPE_TICKET;
    const CDOPER_DEFAULT      = '000000000997';
    const NRORG_DEFAUlT       = '20';

    protected $orderReverse = false;
    protected $orderBy = 'ID';

    const MAIN_TYPE_OF_PRODUCT_CALL = "P";
    const MAIN_TYPE_OF_SUPPORT_CALL = "S";

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var TeknisaEnvironment **/
    protected $environment;
    /** @var GeralRepositories\CpCall */
    protected $cpCallRepository;
    /** @var GeralRepositories\Operador  */
    private $repositoryOperador;
    /** @var \Teknisa\Model\Teknisa\Repositories\OmProduto  */
    private $omProdutoRepository;
    private $now;

    public function __construct(EntityManager $entityManager, TeknisaEnvironment $environment) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;

        $this->cpCallRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'CpCall');
        $this->repositoryOperador = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Operador');
        $this->faturamentoRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Parcnegocio');
        $this->omProdutoRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
    }

    public function getAllSituationDividedByType($eliasSoliserv = 'SER')
    {
        $waitingTicket = Utilities::getWaitingTicket();
        $cdClienteBase = Utilities::getCdClienteBase();
        $allSituation  = Util\InstanceProvider::getRepository()->getOmParcnegocio()->getAllSituation($this->environment->getCurrentOrganizationId(),$waitingTicket);
        list($openedGep, $closedGep) = $this->getBeginAndEndCdfasetram();

        $waitingGEP = " (EXISTS (SELECT CDOPERADOR FROM OPERADOR WHERE CDCLIENTE <> '$cdClienteBase' AND CDOPERADOR = $eliasSoliserv.CDOPERRESP " ;

        $allSituationDividedByType = array(
            'GEP' => [
                'CLOSED' => " $eliasSoliserv.CDFASETRAM = $closedGep OR ($eliasSoliserv.IDSOLIENCE = 'S')",
                'OPENED' => " $eliasSoliserv.CDFASETRAM = $openedGep AND ($eliasSoliserv.IDSOLIENCE <> 'S')",
                'WAITING' => " $waitingGEP AND $eliasSoliserv.CDFASETRAM NOT IN ($closedGep) AND ($eliasSoliserv.IDSOLIENCE <> 'S'))) ",
                'IN_PROGRESS' => " $eliasSoliserv.CDFASETRAM NOT IN ($openedGep,$closedGep) AND ($eliasSoliserv.IDSOLIENCE <> 'S') ",
            ],
            'PM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
            'CRM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
        );
        foreach ($allSituation as $situation) {
            $allSituationDividedByType[$situation['PRODUCT']][$situation['SITUATION']][] = $situation['ID'];
        }
        return $allSituationDividedByType;
    }

    public function getBeginAndEndCdfasetram()
    {
        $paramgeral = $this->cpCallRepository->getBeginAndEndCdfasetram($this->environment->getCurrentOrganizationId());
        return array($paramgeral['CDFATRAMIWEB'],$paramgeral['CDFATRAMENCE']);
    }

    public function findAllOperadorRelationshipCliente($condition, $page = 0, $pageSize = 0){
        $cdCliente = Utilities::getCdClienteBase();
        $userId    = $this->environment->getCurrentUserId();
        $filter    = Utilities::createQueryFiltered($condition);
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->repositoryOperador->findAllOperadorRelationshipCliente($cdCliente, $userId, $filter, $startPos, $finalPos);
    }

    public function findAllOperadorResponsible($condition, $page, $pageSize){
        $filter    = Utilities::createQueryFiltered($condition);
        $filterTeknisa = "";
        $userId = $this->environment->getCurrentUserId();
        $teknisaId = Utilities::getCdClienteBase();

        if(!Utilities::currentUserIsTeknisa()){
            $filterTeknisa = "AND OP.CDCLIENTE IN (
            SELECT CDCLIENTE
            FROM(
                SELECT CLI.CDCLIENTE
                FROM OPERCLIE OCL
                JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = '". $userId ."')
                JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S'))
                UNION
                (SELECT CLI.CDCLIENTE
                FROM OPERADOR OPE
                JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                WHERE OPE.CDOPERADOR = '". $userId ."')
                UNION
                SELECT '". $teknisaId ."' AS CDCLIENTE FROM DUAL
           ) ";
        }
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->repositoryOperador->findAllOperadorResponsible($filter, $filterTeknisa, $finalPos, $startPos);
    }

    /**
     * @param $conditions
     * @param $nrorg
     * @param $nrparcnegocio
     * @param $dtinicompet
     * @param $dtfincompet
     * @return array
     */
    public function faturamento($conditions, $nrparcnegocio, $dtinicompet, $dtfincompet, $page, $pageSize, $orderBy = null)
    {
        $limit = $page * $pageSize;

        $nrorg = $this->environment->getCurrentOrganizationId();
        $orderString = 'ORDER BY DTINICOMPET ASC, NRSEQCONTRATO';
        $offset = ($page - 1) * $pageSize;
        foreach($conditions as &$condition){
            if($condition["columnName"] == "DTFINCOMPET"){
                $condition["operator"] = "<=";
                $condition['value'] = "'" . $condition['value'] . "'";
            }
            if($condition["columnName"] == "DTINICOMPET"){
                $condition["operator"] = ">=";
                $condition['value'] = "'" . $condition['value'] . "'";
            }
        }
        $filter = Utilities::createQueryFiltered($conditions);
        if($orderBy){
            foreach($orderBy as $idx => $order) {
                $orderString = 'ORDER BY ' . $idx . ' ' . $order;
            }
        }
        $dtinicompetFilter = '';
        $dtfincompetFilter = '';
        $dtinicompetFilterMed = '';
        $dtfincompetFilterMed = '';
        if($dtinicompet){
            $dtinicompetFilter = "AND (TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(FAT.DTINICOMPET) OR TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(FAT.DTFINCOMPET))";
            $dtinicompetFilterMed = "AND (TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(MED.DTINICOMPET) OR TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(MED.DTFINCOMPET))";
        }
        if($dtfincompet){
            $dtfincompetFilter = "AND (TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(FAT.DTINICOMPET) OR TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(FAT.DTFINCOMPET))";
            $dtfincompetFilterMed = "AND (TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(MED.DTINICOMPET) OR TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(MED.DTFINCOMPET))";
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getFaturamentoContrato($filter, $dtinicompetFilter, $dtfincompetFilter, $dtinicompetFilterMed, $dtfincompetFilterMed, $nrorg, $nrparcnegocio, $limit, $offset, $orderString);
    }

    public function getContratosForScore($nrparcnegocio) {
        $nrorg = $this->environment->getCurrentOrganizationId();
        $aditionalValues =  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getFinancialForScore($nrorg, $nrparcnegocio);
        $mainScore = Util\InstanceProvider::getRepository()->getOmParcnegocio()->getContratosForScore($nrorg, $nrparcnegocio);
        $mainScore[0]["EXPIRED"] = $aditionalValues[0]["EXPIRED"];
        $mainScore[0]["OPENED"] = $aditionalValues[0]["OPENED"];
        return $mainScore;

    }

    public function getItensContrato($nrfaturacont, $nrorg, $nrparcnegocio, $conditions){
        $filter = Utilities::createQueryFiltered($conditions);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getItensContrato($nrfaturacont, $nrorg, $nrparcnegocio, $filter);
    }

    public function getItemsOrcamento($nrorg, $idadicitemorc, $nrseqcontrato, $nrseqorcamento){
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getItemsOrcamento($nrorg, $idadicitemorc, $nrseqcontrato, $nrseqorcamento);
    }

    public function getNegociacaoContratoCompleto($nrseqcontrato, $filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getNegociacaoContratoCompleto($nrseqcontrato, $filter);
    }

    public function getCompetenciaNegoCont($filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getCompetenciaNegoCont($filter);
    }

    public function getParcelanegocont($filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getParcelanegocont($filter);
    }

    public function getDescperiodonegocont_negociacaocontrato($filters){
        $filter = Utilities::createQueryFiltered($filters);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getDescperiodonegocont_negociacaocontrato($filter);
    }

    public function getElements($nrorg, $idescolhido, $nrseqorcamento, $conditions){
        $filter = Utilities::createQueryFiltered($conditions);
        if(!$nrseqorcamento){
            return false;
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getElements($nrorg, $idescolhido, $nrseqorcamento, $filter);
    }

    public function getEscopoTIpoOrcamento($conditions, $page, $pageSize, $orderBy){
        $orderString= "";
        $filter = Utilities::createQueryFiltered($conditions);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        if($orderBy){
            foreach($orderBy as $idx => $order) {
                $orderString = 'ORDER BY ' . $idx . ' ' . $order;
            }
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getEscopoTIpoOrcamento($filter, $limit, $offset, $orderString);
    }

    public function getContratos($conditions, $nrparcnegocio, $isFilter, $page, $pageSize , $orderBy){
        $nrorg = $this->environment->getCurrentOrganizationId();
        $situacaoFilter = "";
        $orderString= "";
         $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        if($orderBy){
            foreach($orderBy as $idx => $order) {
                $orderString = 'ORDER BY ' . $idx . ' ' . $order;
            }
        }
        if(!$isFilter){
            $situacaoFilter = "AND fct_contrato.idsituacaocontrato = 'A'";
        }
        foreach ($conditions as &$condition) {
            if($condition['columnName'] == "NMTIPOORCAMENTO" || $condition['columnName'] == "NRTIPOORCAMENTO"){
                $condition['columnName'] = 'ESCOPO.' . $condition['columnName'];
            }else{
                $condition['columnName'] = 'fct_contrato.' . $condition['columnName'];
            }

        }
        $filter = Utilities::createQueryFiltered($conditions);
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->getContratos($nrorg, $filter,$nrparcnegocio, $situacaoFilter, $limit, $offset, $orderString);
    }

    /**
     * @param $conditions
     * @param $nrparcnegocio
     * @return array
     */
    public function findProdutosContratados($conditions, $nrparcnegocio)
    {
        $filter = Utilities::createQueryFiltered($conditions);
        return $this->omProdutoRepository->findProdutosContratados($filter, $nrparcnegocio);

    }

    public function getOrganizationScore(){
        $minRevenue = Utilities::getClassificationRevenueValueMinParameter();
        $organizationScore = InstanceProvider::getRepository()->getLocalOrganization()->getOrganizationScore($minRevenue);
        return $organizationScore;
    }

    public function findCallsByType($orgId)
    {
        $situationFilters = $this->getAllSituationDividedByType();

        $result  =  Util\InstanceProvider::getRepository()->getOmParcnegocio()->findCallsByType($situationFilters, $orgId);

        return $result;
    }

    public function financeiro($nrparcnegocio, $dtemissao, $dtvencimento, $dtpagamento, $vencboleto)
    {
        $emissaoFilter = "";
        $vencFilter = "";
        $pagFilter = "";
        if($dtemissao){
            $emissaoFilter = "  AND NF.DTEMISSAO BETWEEN '" . $dtemissao[0] . "' AND '" . $dtemissao[1] . "'";
        }

        if($dtvencimento){
            if($vencboleto){
                $vencFilter = "AND ((CR.DTATUAVENREC < TRUNC(SYSDATE)) AND (BO.IDSTATBREC IN ('A','E')) 
                OR (CR.DTATUAVENREC BETWEEN '" . $dtvencimento[0] . "' AND '" . $dtvencimento[1] . "'))";
            }else{
                $vencFilter =  "AND CR.DTATUAVENREC BETWEEN '" . $dtvencimento[0] . "' AND '" . $dtvencimento[1] . "'";
            }
        }
        if($dtpagamento){
            $pagFilter = " AND ((BR.DTBAIXAREC BETWEEN '". $dtpagamento[0] . "' AND '" . $dtpagamento[1] ."'))";
        }
        return  Util\InstanceProvider::getRepository()->getOmParcnegocio()->scoreReportGetFinanceiro($nrparcnegocio, $emissaoFilter, $vencFilter, $pagFilter);
    }

    /**
     * @param array $arrayRevenues
     * @param EntidadeOrganizacao $org
     * @return array
     */
    private function setsInitautoUpdateNrscores(array $arrayRevenues, EntidadeOrganizacao $org)
    {
        $nrorg = $org->getNrorg();
        if (empty($arrayRevenues[$nrorg])) {
            $soma = 1;
            $allNrorg = array($nrorg);
            $revenue = 0.0;
        } else {
            $row = $arrayRevenues[$nrorg];
            $soma = empty($row['SOMA']) ? '1' : $row['SOMA'];
            $allNrorg = $row['ALL_NRORG'];
            $revenue = floatval($arrayRevenues[$nrorg]['FATURAMENTO']);
        }
        return array($allNrorg, $soma, $revenue);
    }

    /**
     * @param $allNrorg
     * @param $arrayRevenues
     * @return float
     */
    private function getTotalRevenues($allNrorg, $arrayRevenues)
    {
        $revenues = 0.0;
        foreach ($allNrorg as $nrorg) {
            if (!empty($arrayRevenues[$nrorg])) {
                $revenues += floatval($arrayRevenues[$nrorg]['FATURAMENTO']);
            }
        }
        return $revenues;
    }

    /**
     * @param array $arrayOmParamOrg
     * @param EntidadeOrganizacao $org
     * @param $revenues
     * @param \OrganizationsManagement\Factory\OmParamOrg $omParamOrgFactory
     * @param $revenue
     * @param \DateTime $now
     * @return mixed|EntidadeOmParamOrg
     */
    /**
     * @param $maintenanceType
     * @return array
     */
    private function getresultRevenuesIndexed($maintenanceType)
    {
        $resultRevenues = $this->omProdutoRepository->getCalculatedNrscoreAndRevenues($maintenanceType);
        $arrayRevenues = array();
        foreach ($resultRevenues as $fatu) {
            $fatu['ALL_NRORG'] = Utilities::setSelectMultiInQuery($fatu['ALL_NRORG']);
            $arrayRevenues[$fatu['NRORG']] = $fatu;
        }
        return $arrayRevenues;
    }

    private function getAllFinancial()
    {
        $allFinancial = InstanceProvider::getRepository()->getOmParcnegocio()->getAllFinancialForScore();
        $arrayAllFinancial= array();
        foreach ($allFinancial as $fatu) {
            $arrayAllFinancial[$fatu['NRPARCNEGOCIO']] = $fatu;
        }
        return $arrayAllFinancial;
    }

    /**
     * @return EntidadeOmParamOrg[]
     */
    private function getOmParamOrgEntitiesIndexed()
    {
        /** @var EntidadeOmParamOrg[] $omParamOrgEntities */
        $omParamOrgEntities = InstanceProvider::getRepository()->getOmparamorg()->findBy(array());
        /** @var EntidadeOmParamOrg[] $arrayOmParamOrg */
        $arrayOmParamOrg = array();
        foreach ($omParamOrgEntities as $omParamOrgEntity) {
            $arrayOmParamOrg[$omParamOrgEntity->getNrorg()] = $omParamOrgEntity;
        }
        return $arrayOmParamOrg;
    }

    /**
     * @param $orderBy
     * @param $conditions
     * @return string
     */
    private function setOrderByInScore($orderBy, $conditions)
    {
        if (empty($orderBy)) {
            $conditions .= ' ORDER BY TO_NUMBER(NRSCORE) DESC';
        } else {
            foreach ($orderBy as $idx => $order) {
                if ($idx == 'VRCNPJFATU' || $idx == 'VRTOTALFATU') {
                    $idx = "TO_NUMBER($idx)";
                }
                $conditions .= ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        return $conditions;
    }

    /**
     * @param $isPrimary
     * @param $conditions
     * @return string
     */
    private function setFiltersAdditional($isPrimary, $conditions)
    {
        if ($isPrimary && $isPrimary == "S") {
            $conditions .= "AND NRORGPRINCIPAL IS NOT NULL";
        } else if ($isPrimary && $isPrimary == "P") {
            $conditions .= "AND NRORGPRINCIPAL IS NULL";
        }
        return $conditions;
    }


}
