<?php
namespace OrganizationsManagement\Util\ORM;
use Doctrine\DBAL\Platforms\AbstractPlatform;
use Doctrine\DBAL\Types\ConversionException;
use Doctrine\DBAL\Types\DateType as DoctrineDateType;
use Setforms2\Shared\ORM\DateTime as SetformsDateTime;

class DateType extends DoctrineDateType
{
    const DATE_FORMAT = 'd/m/Y H:i:s';

    public function convertToPHPValue($value, AbstractPlatform $abstractPlatform)
    {
        if($value === null || $value instanceof SetformsDateTime){
            return $value;
        }

        $formatString = self::DATE_FORMAT;
        if(strpos($formatString, '00:00:00') > 0){
            $value = $this->replaceHourToZeroValues($value);
        }

        $val = SetformsDateTime::createFromFormat($formatString, $value);

        if( ! $val){
            throw ConversionException::conversionFailedFormat($value, $this->getName(), $formatString);
        }

        return $val;
    }

    private function replaceHourToZeroValues($value)
    {
        return preg_replace('/\d\d:\d\d:\d\d/', '00:00:00', $value);
    }

    public function convertToDatabaseValue($value, AbstractPlatform $platform)
    {
        return ($value !== null)
            ? $value->format(self::DATE_FORMAT) : null;
    }
} 