<?php
namespace OrganizationsManagement\Util;

use Zeedhi\Framework\Session\Session;
use OrganizationsManagement\Util\Messages as Messages;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use MongoDB\BSON\UTCDateTime as MongoDate;

class Utilities {
    const ID_CLIENTE_DEVOPS = "cd7e82153be60953002055f9b19a60b3";
    const SECRET_CLIENTE_DEVOPS = "do25449996c21f780fe30badd6522040";
    const ID_CLIENTE_LIBRARIES = "cd7e85178153by60953518945f9hakt19a50a4";
    const SECRET_CLIENTE_LIBRARIES = "lib5449996c21f780fe30badd6521931a5sac9";
    const SECRET_CLIENTE_LOGIN = '107913b19f7d84a3b0eff2d3b0ca9b3f';

    const GERAL_ENTITY_DIR = '\Teknisa\Model\Geral\Entities\\';
    const INTEGRACAO_ENTITY_DIR = '\Teknisa\Model\Integracao\Entities\\';
    const TEKNISA_ENTITY_DIR = '\Teknisa\Model\Teknisa\Entities\\';
    const SERVICES_ENTITY_DIR = '\Teknisa\Model\Services\Entities\\';
    const AVALIACAO_ENTITY_DIR = '\Teknisa\Model\Avaliacao\Entities\\';
    const CRM_ENTITY_DIR = "\Teknisa\Model\Crm\Entities\\";
    const FATURAMENTO_ENTITY_DIR = "\Teknisa\Model\Faturamento\Entities\\";

    const LOGIN_CRYPT_STRING = '1460_loginManagement';

    protected $smURL;
    protected $session;
    protected $devOpsURL;
    protected $smIntegrationRequestType;
    protected $tmpLogPath;
    protected $exportStructurePath;
    protected $baseUri;


    public function __construct(Session $session, $devOpsURL, $smURL, $smIntegrationRequestType, $tmpLogPath, $exportStructurePath, $baseUri){
        $this->session = $session;
        $this->devOpsURL = $devOpsURL;
        $this->smURL = $smURL;
        $this->tmpLogPath = $tmpLogPath;
        $this->exportStructurePath = $exportStructurePath;
        $this->smIntegrationRequestType = $smIntegrationRequestType;
        $this->baseUri = $baseUri;
    }

    static public function dateStringToDateObject($dateString, $divider = '/'){
        return new \DateTime(implode($divider, array_reverse(explode($divider, $dateString))));
    }

    static public function removeCaracteresEspeciais($texto){
        return preg_replace('/[^a-zA-Z0-9]/', '', $texto);
    }

    public static function getWaitingTicket(){
        return InstanceManager::getInstance()->getParameter('WAITING_TICKET');
    }

    public static function getCdClienteBase() {
        return InstanceManager::getInstance()->getParameter('cdClienteBase');
    }

    static public function preencheStringComZero($string, $numCaracteres){
        return str_pad($string, $numCaracteres, '0', STR_PAD_LEFT);
    }
    
    public static function parseDataSourceZeedhiNext($data){
        header('Content-type: application/json; charset=utf-8');
        header("Access-Control-Allow-Origin: *");
        header("Access-Control-Allow-Credentials: false");
        header("Access-Control-Allow-Methods: GET,HEAD,OPTIONS,POST,PUT");
        header("Access-Control-Allow-Headers: *");
        $data = json_encode($data);
        $response = '{"pagination" : {"total" : 10, "page" : 1, "limit" : 10},"data" :'. $data. '}';
        
        echo($response);
        die;
    }

    public function obtemTokenDevOpsDaSessao(){
        return $this->session->get("tokenDevops") ? $this->session->get("tokenDevops") : null;
    }

    public function defineTokenDevOpsNaSessao($token){
        $this->session->get("tokenDevops", $token);
    }

    public static function getCurrentUserId() {
        return InstanceProvider::getLibEnvironment()->getCurrentUserId();
    }

    public static function currentUserIsTeknisa() {
        $user = self::getCurrentUser();
        return self::getCdClienteBase() == $user->getCdcliente();
    }

    /**
     * @return \Teknisa\Model\Geral\Entities\Operador
     */
    public static function getCurrentUser() {
        $entityManager = InstanceProvider::getEntityManager();
        return $entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')->find(
            array(
                'cdoperador' => self::getCurrentUserId(),
                'nrorg' => InstanceProvider::getLibEnvironment()->getCurrentOrganizationId(),
            )
        );
    }

    public function getDevOpsURL(){
        return $this->devOpsURL;
    }
    
    public function getSmUrl(){
        $nrorg = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId() ?: self::getTeknisaNrorgParameter();
        $requesterUrl = InstanceProvider::getRepository()->getOmIntegracoes()->getSMIntegrationUrl($nrorg)[0]["DSURLSM"];
        if(empty($requesterUrl)) {
            throw new \Exception(Messages::ERR_EMPTY_URL);
        }
        $requesterUrl .= $this->baseUri;
        return $requesterUrl;
    }

    /**
     * @param $array
     * @param String $key
     * @return array
     */
    public static function getArrayFromArrayAssociative( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key];
        }
        return $result;
    }
    
    public function getSmIntegrationRequestType(){
        return $this->smIntegrationRequestType;
    }

    public function obtemTokenLoginDaSessao(){
        return $this->session->get("tokenLogin") ? $this->session->get("tokenLogin") : null;
    }

    public function defineTokenLoginNaSessao($token){
        $this->session->set("tokenLogin", $token);
    }

    public function chamaWebService($token, $tipoRequisicao, $dados, $dataSetResultPos, $chavePrivada, $url, $timeout = 180) {
        set_time_limit($timeout);
        
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $chavePrivada;
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($dados));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $tipoRequisicao);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($dados));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
        $resultado = curl_exec($ch);

        $resultado = json_decode($resultado, true);
        $error = curl_error($ch);
        curl_close($ch);
        if (!empty($resultado['error'])) {
            throw new \Exception($resultado['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else {
            return isset($resultado['dataset'][$dataSetResultPos]) ? $resultado['dataset'][$dataSetResultPos] : null;
        }
    }

    public function criaMetodoSetterPorString($valor){
        return "set" . str_replace("_", "", ucwords(strtolower($valor), "_"));
    }

    static public function geraCriptografiaSenhaDelphi($password){
        $url = 'http://security.teknisa.com/comwebservice/service.asmx?wsdl';

        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );

        $params = array(
            'rdm'    => 'Bib_html.rdmDBGeralHtm',
            'method' => 'GetCriptoPass',
            'args'   => $args,
        );

        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }

        return $encryption;
    }

    
    public static function replaceSpecialCharacter($string){
        $unwanted_array = array(    'Š'=>'S', 'š'=>'s', 'Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A', 'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E',
                            'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I', 'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U',
                            'Ú'=>'U', 'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss', 'à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a', 'å'=>'a', 'æ'=>'a', 'ç'=>'c',
                            'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i', 'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o',
                            'ö'=>'o', 'ø'=>'o', 'ù'=>'u', 'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y' );
        return strtr( $string, $unwanted_array );
    }
    
    private static function elementsMenuToArray($allElements, $groupMenu, $parentId = null, $ProductId){
        foreach($groupMenu as $menuElement){
            if ((isset($menuElement->isVisible) && $menuElement->isVisible === true) || !isset($menuElement->isVisible)){
                $element = array(
                    "UNIQUE_ID"    => isset($menuElement->id) ? $menuElement->id : false,
                    "LABEL"        => isset($menuElement->label) ? $menuElement->label : null,
                    "EX_PRODUCT_ID"=> isset($menuElement->devOpsId) ? $menuElement->devOpsId : null,
                    "PROJECT_ID"   => $ProductId,
                    "TYPE"         => !!$parentId ? "M" : "G",
                    "PARENT_ID"    => $parentId,
                    "NRORDEM"      => isset($menuElement->order) ? $menuElement->order : 0
                );
                array_push($allElements, $element);
                if (isset($menuElement->menus) && count(get_object_vars($menuElement->menus)) > 0 && isset($menuElement->id)) {
                    $allElements = self::elementsMenuToArray($allElements, $menuElement->menus, $menuElement->id, $ProductId);
                }
            }
        }
        return $allElements;
    }
    
    public function getElementsMenu($urlJsonFile, $productId, $elementsContainer = array()){
        if(empty($elementsContainer)) {
            $elementsContainer = $this->getElementsContainer($urlJsonFile);
        }
        if ($elementsContainer){
            $elementsContainer = self::convertToObject($elementsContainer);
            return self::elementsMenuToArray([], $elementsContainer, null, $productId);
        } else {
            return null;
        }
    }

    /**
     * @param $urlJsonFile
     * @return array
     * @throws \Exception
     */
    public function getElementsContainer($urlJsonFile){
        $productUrl = explode('/', $urlJsonFile);
        $route = '/' . array_pop($productUrl);
        $url = join('/', $productUrl);
        return $this->getProductMenu($url, $route);
    }
    
     public static function planningMenuTreeGrid ($elementsSorted, $elementsToSort){
        if (!empty($elementsToSort) && !empty($elementsSorted)){
            $menuElementsSorted = self::sortMenuElements($elementsSorted, $elementsToSort);
            return self::generateMenuElementsTreeGrid($menuElementsSorted);
        } else if (!empty($elementsToSort)){
             return self::generateMenuElementsTreeGrid($elementsToSort);
        } else if (!empty($elementsSorted)){
            return self::generateMenuElementsTreeGrid($elementsSorted);
        } else {
            return array();
        }
        
    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }
    
    private static function sortMenuElements($elementsSorted, $elementsToSort){
        $menuElementsSorted = array();
        foreach($elementsSorted as $element){
            $key = array_search($element['UNIQUE_ID'], array_column($elementsToSort, 'UNIQUE_ID'));
            if ($key > -1){
                array_push($menuElementsSorted, $elementsToSort[$key]);
            }
        }
        foreach($elementsToSort as $element){
            $key = array_search($element['UNIQUE_ID'], array_column($menuElementsSorted, 'UNIQUE_ID'));
            if ($key === false){
                 array_push($menuElementsSorted, $element);
            }
        }
        return $menuElementsSorted;
    }
    
    public static function generateMenuElementsTreeGrid($elements) {
        $generatedMenuElements = [];
        foreach ($elements as $element) {
            array_push ($generatedMenuElements, array (
                "ID"          => isset($element["ID"]) ? $element["ID"] : '',
                "ATIVO"       => isset($element["ATIVO"]) ? $element["ATIVO"] : '',
                "PRODUCT_ORG" => isset($element["NRPRODUTO_ORG_ID"]) ? $element["NRPRODUTO_ORG_ID"] : '',
                "PROJECT_ID"  => isset($element["NRPRODUTO_ID"]) ? $element["NRPRODUTO_ID"] : $element["PROJECT_ID"],
                "DTVALIDADE"  => isset($element["DTVALIDADE"]) ? $element["DTVALIDADE"] : '',
                "CDSISTEMA"   => isset($element["CDSISTEMA"]) ? $element["CDSISTEMA"] : '',
                "NMSISTEMA"   => isset($element["NMSISTEMA"]) ? $element["NMSISTEMA"] : '',
                "OPERINCLUSAO"   => isset($element["OPERINCLUSAO"]) ? $element["OPERINCLUSAO"] : '',
                "OPERULTATU"   => isset($element["OPERULTATU"]) ? $element["OPERULTATU"] : '',
                "DTULTATU"   => isset($element["DTULTATU"]) ? $element["DTULTATU"] : '',
                "DTINCLUSAO"   => isset($element["DTINCLUSAO"]) ? $element["DTINCLUSAO"] : '',
                "DSOBSERVACAO"   => isset($element["DSOBSERVACAO"]) ? $element["DSOBSERVACAO"] : '',
                "NREXTERNAL_PRODUCT_ID"  => isset($element["NREXTERNAL_PRODUCT_ID"]) ? $element["NREXTERNAL_PRODUCT_ID"] : '',
                "UNIQUE_ID"   => $element["UNIQUE_ID"],
                "PARENT_ID"   => $element["PARENT_ID"],
                "TYPE"        => $element["PARENT_ID"] ? "M" : "G",
                "LABEL"       => $element["LABEL"] ? $element["LABEL"] : "[SEM_LABEL]",
                "NAME"        => str_replace(" ", "_", strtoupper(self::replaceSpecialCharacter($element["LABEL"]))),
                "IS_RELATED_TO_CONTRACT" => $element['IS_RELATED_TO_CONTRACT']
            ));
        }
        
        self::fixChildrenWithoutParentInTreeGrid($generatedMenuElements, 'PARENT_ID', 'ID');
        
        return $generatedMenuElements;
    }
    
    private static function fixChildrenWithoutParentInTreeGrid(&$generatedMenuElements, $indexReferenceToParent, $indexParentPrimaryProperty){
        foreach ($generatedMenuElements as &$element) {
            if (array_search($element[$indexReferenceToParent], array_column($generatedMenuElements, $indexParentPrimaryProperty)) === false){
                $element[$indexReferenceToParent] = null;
            }
        }
    }

    /**
     * The Tree Grid has a problem: any rows whose parent elements don't exist in the dataset
     * will not show up at all.
     * So, this method sets nonexistent parent elements as null to avoid this problem.
     * @param array $values
     * @param string $indexPrimary
     * @param string $indexParent
     * @return mixed
     */
    public static function fixParentsForTreeGrid(array $values,$indexPrimary = 'ID',$indexParent = 'PARENT')
    {
        $aux = array();
        foreach ($values as $item) {
            array_push($aux, $item[$indexPrimary]);
        }
        foreach ($values as &$item) {
            if (!in_array($item[$indexParent], $aux)) {
                $item[$indexParent] = null;
            }
        }
        return $values;
    }
    
    private static function getErrorJson($error){
        switch($error) {
            case JSON_ERROR_DEPTH:
                return Messages::JSON_ERROR_DEPTH;
                break;
            case JSON_ERROR_STATE_MISMATCH:
                return Messages::JSON_ERROR_STATE_MISMATCH;
                break;
            case JSON_ERROR_CTRL_CHAR:
                return Messages::JSON_ERROR_CTRL_CHAR;
                break;
            case JSON_ERROR_SYNTAX:
                return Messages::JSON_ERROR_SYNTAX;
                break;
            case JSON_ERROR_UTF8:
                return Messages::JSON_ERROR_UTF8;
                break;
            default:
                return Messages::JSON_ERROR_UNKNOWN;
        }
    }
    
    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false){
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }
    
    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if($name == 'NRLICENCAUSO'){
                $operator = 'LIKE';
            }
            //(!is_null($values) && $operator == '=') ->Gambi to fix framework bugs :(
            if ((($name != '*' && !empty($values)) || $values == 0) && !(empty($values) && $operator == '=') && !is_null($values)) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode("','", $values) . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE' || $operator == 'LIKE_I') {
                        $operator = 'LIKE';
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode(' OR ', $likes);
                        $filterSQL .= " AND ( $filterLikeAll )";
                    } else if ($operator == 'LIKE') {
                        $filterSQL .= " AND $name $operator $values ";
                    } else if ($operator == 'IS_NULL') {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name IS $values ";
                        } else {
                            $filterSQL .= " AND $name IS $values ";
                        }
                    } else if ($operator == '='){
                        if(!empty($tableAlias)){
                            $filterSQL .= " AND $tableAlias.$name $operator '$values' ";
                        } else {
                            $filterSQL .= " AND $name $operator '$values' ";
                        }
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $values = "UPPER('" . $mappedLikeAllValues . "')";

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if (empty($tableAlias)) {
                                        $likes[] = "UPPER($column) LIKE $values";
                                    } else {
                                        $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                    }
                                }
                                $filterLikeAll = implode(' OR ', $likes);
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode("','", $mappedLikeAllValues) . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values ";
                        } else {
                            $filterSQL .= " AND $name $operator $values ";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }

    public function getLibrariesToken(){
        return $this->session->get("tokenLib") ? $this->session->get("tokenLib") : null;
    }

    public function setLibrariesToken($token){
        $this->session->set("tokenLib", $token);
    }

    public function getLibToken($url) {
        $param = array(
            'CLIENT_ID' => self::ID_CLIENTE_LIBRARIES,
            'CLIENT_SECRET' => self::SECRET_CLIENTE_LIBRARIES
        );
        $token = $this->chamaWebService(null, '/lib_requestToken', $param, 'token',
            self::SECRET_CLIENTE_LIBRARIES, $url);
        $this->setLibrariesToken($token[0]);
        return $token[0];
    }

    public function getProductMenu($url, $route) {
        $token = $this->getLibToken($url);
        $filter = array(
            "requestType" => "FilterData",
            "filter" => array( array(
                'name' => 'CONCATENATE_GROUP_MENU',
                'operator' => '=',
                'value' => true,
            ) )
        );
        return $this->chamaWebService($token, $route, $filter, 'menu', self::SECRET_CLIENTE_LIBRARIES, $url);
    }

    public static function convertToObject($array) {
        $object = new \stdClass();
        
        if (!empty($array)){
            foreach ($array as $key => $value) {
                if (is_array($value)) {
                    $value = self::convertToObject($value);
                }
                $object->$key = $value;
            }
        }
        return $object;
    }

    public static function zeedhiEncrypt($text, $salt = self::LOGIN_CRYPT_STRING) {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    str_pad($salt, 32, '\0'),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }

    public static function getRemoteFileSize($url) {
        $data = get_headers($url, true);
        if (isset($data['Content-Length'])) {
            return (int)$data['Content-Length'];
        }
        return null;
    }

    public static function downloadAndRemoveFile($fileDir, $fileName, $fileType = 'text/plain', $isRemoveFile = true) {
        if(strpos($fileDir, 'http') === false) {
            $fileSize = filesize($fileDir);
        } else {
            $fileSize = self::getRemoteFileSize($fileDir);
            $header   = array (
                "apiKey: " . self::getFileServerKeyParameter()
            );
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $fileDir);
            curl_setopt($ch, CURLOPT_POST, false);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);

            $resultado = curl_exec($ch);
            $error = curl_error($ch);
            curl_close($ch);

            if (!empty($error)) {
                throw new \Exception($error);
            } else {
                $fileDir = self::getTempLogPathParameter() . DIRECTORY_SEPARATOR . $fileName;
                file_put_contents($fileDir, $resultado);
            }
        }

        header('Accept-Ranges: none');
        header("Content-Type: " . $fileType);
        header("Content-Length: " . $fileSize);
        header("Content-Disposition: attachment; filename=" . $fileName);
        readfile($fileDir);

        if($isRemoveFile) {
            unlink($fileDir);
        }
        exit;
    }

    public static function getOrgDbUserName($orgId) {
        return 'USR_ORG_' . $orgId;
    }

    public function getLogFileDir($fileName) {
        return $this->tmpLogPath . '/' . $fileName;
    }

    public function getExportStructurePath() {
        return $this->exportStructurePath;
    }
    
    public static function generateNewCode($nrorg, $stcodigo, $intervalo = 1, $tamanhoColuna = 10, $numerico = false,
                                           $commit = true, $connection = null, $isNewCodeAlpha = false) {
        $novocodigo = self::executeProcedure($stcodigo, $nrorg, $intervalo, $connection, $commit, $isNewCodeAlpha);
        return self::prepareValue($novocodigo, $tamanhoColuna, $numerico);
    }

    protected static function executeProcedure($stcodigo, $nrorg, $intervalo, $connection = null, $commit = true, $isNewCodeAlpha = false) {
        if($isNewCodeAlpha) {
            $procedure = new Procedure('NOVO_CODIGO_ALFA');
            $stcodigo .= 'ALFA';
        } else if($commit) {
            $procedure = new Procedure('NOVO_CODIGO');
        } else {
            $procedure = new Procedure('NOVO_CODIGO_SC');
        }

        if($connection) {
            $procedure->setConnection($connection);
        }
        $procedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $stcodigo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE', Param::PARAM_INPUT, $intervalo, Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_INT));
        $novocodigo = $procedure->call();
        return $novocodigo['P_SEQUENCIAL'];
    }

    protected static function prepareValue($novocodigo, $tamanhoColuna, $numerico) {
        return $numerico === true ? (int)$novocodigo :  substr($novocodigo, -1 * $tamanhoColuna);
    }
    
    public static function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $limit . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $limit;
            }
        }
        return $query;
    }

    public function convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName) {
        $zmartFilterCriteria = new ZmartFilterCriteria($dataSourceName);
        foreach($conditions as $condition) {
            $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
            switch (true) {
                case $this->isGroupBy($condition):
                    $zmartFilterCriteria->setGroupBy($name);
                    break;
                case $this->isAggregateFunction($condition):
                    $zmartFilterCriteria->addAggregateFunction($this->getLabelOrName($condition), $condition['operator'], $name);
                    break;
                case $this->isOrderBy($condition):
                    $zmartFilterCriteria->addOrderBy($name['name'], $name['direction']);
                    break;
                default:
                    $zmartFilterCriteria->addCondition($name, $condition['operator'], $condition['value']);
            }
        }
        return $zmartFilterCriteria;
    }

    protected function isGroupBy($condition) {
        return $condition['operator'] === ZmartFilterCriteria::GROUP_BY;
    }

    protected function isAggregateFunction($condition) {
        $functions = array(
            ZmartFilterCriteria::SUM,
            ZmartFilterCriteria::AVG,
            ZmartFilterCriteria::MIN,
            ZmartFilterCriteria::MAX,
            ZmartFilterCriteria::COUNT
        );
        return in_array($condition['operator'], $functions);
    }

    protected function isOrderBy($condition) {
        return $condition['operator'] === ZmartFilterCriteria::ORDER_BY;
    }

    protected function getLabelOrName(array $condition) {
        $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
        return isset($condition['label']) ?:$name;
    }

    public static function getMongoDate($date, $format = 'd/m/Y') {
        if(!($date instanceof \DateTime)) {
            $timezone = new \DateTimeZone('UTC');
            $date = \DateTime::createFromFormat($format, $date, $timezone);
        }
        return new MongoDate($date->getTimestamp() * 1000);
    }

    public static function getStructureUpdateIgnoredErrorsParameter() {
        return InstanceManager::getInstance()->getParameter('structureUpdateIgnoredErrors');
    }

    public static function getZmartNrorgParameter() {
        return InstanceManager::getInstance()->getParameter('ZmartNrorg');
    }

    public static function getTeknisaNrorgParameter() {
        return InstanceManager::getInstance()->getParameter('teknisaNrorg');
    }

    public static function getZmartPasswordOperadorParameter() {
        return InstanceManager::getInstance()->getParameter('ZmartPasswordOperador');
    }

    public static function getFileServerFilesUrlParameter() {
        return InstanceManager::getInstance()->getParameter('fileServerFilesUrl');
    }

    public static function getFileServerKeyParameter() {
        return InstanceManager::getInstance()->getParameter('fileServerKey');
    }
    public static function getMaintenanceType() {
        return InstanceManager::getInstance()->getParameter('maintenanceType');
    }

    public static function getDefaultNrproposfasParameter() {
        return InstanceManager::getInstance()->getParameter('defaultNrproposfas');
    }

    public static function getTempLogPathParameter() {
        return InstanceManager::getInstance()->getParameter('tempLogPath');
    }

    public static function getDefaultNrverpropfasParameter() {
        return InstanceManager::getInstance()->getParameter('defaultNrverpropfas');
    }

    public static function getBaseUriParameter() {
        return InstanceManager::getInstance()->getParameter('baseUri');
    }

    public static function getVariableMeasureEmailToSendParameter() {
        return InstanceManager::getInstance()->getParameter('variableMeasureEmailToSend');
    }

    public static function getContractDaysToAddParameter() {
        return InstanceManager::getInstance()->getParameter('contractDaysToAdd');
    }
    
    public static function getCdFaseTramFilterParameter() {
        return InstanceManager::getInstance()->getParameter('cdFaseTramFilter');
    }

    public static function getFilePathToBeSentDelphiParameter() {
        return InstanceManager::getInstance()->getParameter('filePathToBeSentDelphi');
    }

    public static function getAllowedFilesToBeSentParameter() {
        return InstanceManager::getInstance()->getParameter('allowedFilesToBeSent');
    }

    public static function getAllowedFilesToBeSentParameterArray() {
        $allowedFiles = InstanceManager::getInstance()->getParameter('allowedFilesToBeSent');
        return explode(',', $allowedFiles);
    }

    public static function strpos_arr($haystack, $needle)
    {
        if(!is_array($needle)) $needle = array($needle);
        foreach($needle as $what) {
            if(($pos = strpos($haystack, $what))!==false) return $pos;
        }
        return false;
    }

    public static function deleteDirectory($dir)
    {
        if (!file_exists($dir)) {
            return true;
        }
        if (!is_dir($dir)) {
            return unlink($dir);
        }

        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') {
                continue;
            }
            if (!self::deleteDirectory($dir . DIRECTORY_SEPARATOR . $item)) {
                return false;
            }

        }
        return rmdir($dir);
    }

    public static function copyDir($origin, $target)
    {
        //O nome das pastas tem de terminar em "/" ou ""(win).
        if (is_dir($origin)) {
            if (is_dir($target)) {
                if ($dir = opendir($origin)) {
                    while (($file = readdir($dir)) !== false) {
                        if (!is_dir($origin.$file)) {
                            copy($origin.$file, $target.$file);
                        }
                    }
                    closedir($dir);
                }
            } else {
                throw new \Exception("A pasta \"$target\" não existe.");
            }
        }else{
            throw new \Exception("A pasta \"$origin\" não existe.");
        }
    }

    public static function deleteAllFilesInFolder($path)
    {
        $files = glob($path . '/*');
        foreach($files as $file) {
            if(is_file($file)) {
                unlink($file);
            }
        }
    }

    public static function toZipDir($pathDir, $zipCreated)
    {
        $zip = new \ZipArchive;
        if($zip->open($zipCreated, \ZipArchive::CREATE ) === TRUE) {
            // Store the path into the variable
            $dir = opendir($pathDir);
            while($file = readdir($dir)) {
                if(is_file($pathDir.$file)) {
                    $zip->addFile($pathDir.$file, $file);
                }
            }
            $zip->close();
        }
    }

    public static function dirToArray($dir) {

        $result = array();

        $cdir = scandir($dir);
        foreach ($cdir as $key => $value)
        {
            if (!in_array($value,array(".","..")))
            {
                if (is_dir($dir . DIRECTORY_SEPARATOR . $value))
                {
                    $result[$value] = dirToArray($dir . DIRECTORY_SEPARATOR . $value);
                }
                else
                {
                    $result[] = $value;
                }
            }
        }

        return $result;
    }

    public static function setSelectMultiInQuery($stringValue, $int = false, $delimiter = ',')
    {
        if (empty($stringValue)) {
            $value = array();
        } else {
            if ($int) {
                $value = array_map('intval', explode($delimiter, $stringValue));
            } else {
                $value = explode($delimiter, $stringValue);
            }
        }
        return array_values($value);
    }

    public static function setDataSetDefault(Request $request, Response $response, array $dataSet)
    {
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet(explode('/', $request->getRoutePath())[1], $dataSet));
    }

    public static function getClassificationRevenueValueMinParameter() {
        return InstanceManager::getInstance()->getParameter('classificationRevenueValueMin');
    }

    public static function fixBugWhereINQuery(array $allProjectIds, $column, $isNotIn = false, $isString = false)
    {
        $arrays = array_chunk($allProjectIds, 900);
        $allProjectsString = '';
        $in = $isNotIn ? 'NOT IN' : 'IN';
        $connection = $isNotIn ? 'AND' : 'OR';
        if ($isString) {
            $glue = "','";
            $start = "'";
            $end = "'";
        } else {
            $glue = ',';
            $start = '';
            $end = '';
        }
        foreach ($arrays as $array) {
            $allProjectsString .= " $column $in (" . Utilities::getStringForArray($array, $glue, $start, $end) . ") $connection";
        }
        $allProjectsString .= " $column $in ($start 0 $end) ";
        return $allProjectsString;
    }

    /**
     * @param $array
     * @param string $glue
     * @param string $start
     * @param string $end
     * @param string $default
     * @return string
     */
    public static function getStringForArray($array,$glue = ',',$start = '',$end = '',$default = ''){
        $return = $default;
        if(!empty($array)) {
            $return = $start . implode($glue, $array) . $end;
        }
        return $return;
    }

    public static function getReversedDate($date, $inFormat = 'd/m/Y', $outFormat = 'Ymd') {
        $date = \DateTime::createFromFormat($inFormat, $date);
        return $date->format($outFormat);
    }
}